@@ -60,7 +60,7 @@ umat <- function(formula, relmat, data, addmat){
     }
     tabRec <- table(data$record)
     if(length(tabRec) > 1){
-      if( var(tabRec) > 0 ){stop("The eigen decomposition only works for balanced datasets. Please set to FALSE.", call. = FALSE)}
+      if( var(tabRec) > 0 ){stop("The eigen decomposition only works for balanced datasets. Please ensure you fill the dataset to make it balanced for the 'relmat' terms or set to FALSE.", call. = FALSE)}
     }
     data$recordF <- as.factor(data$record)
     nLev <- length(levels(data$recordF))
@@ -99,7 +99,8 @@ umat <- function(formula, relmat, data, addmat){
     rownames(D) <- colnames(D) <- rownames(relmat[[iProv]])
     rownames(U) <- colnames(U) <- rownames(relmat[[iProv]])
     common <- intersect(colnames(U), colnames(Z))
-    Ul[[iProv]]<-U[common,common]
+    Ul[[iProv]]<- U[common,common]
+    # Ul[[iProv]]<- (t(Z[,common]%*%t(U[common,common]))%*%Z[,common])/4
     Dl[[iProv]]<-D[common,common]# This will be our new 'relationship-matrix'
     Zu[[iProv]] <- Z[,common]
   }
@@ -112,7 +113,7 @@ umat <- function(formula, relmat, data, addmat){
   # UBind <- do.call(Matrix::bdiag, Ul)
   # part1 <- ZuBind%*%UBind%*%t(ZuBind)
   # W0 <- part0 * part1
-  return(list(Utn=Utn, D=Dl, U=Ul, RRt=ZrZrt, effect=idProvided))
+  return(list(Utn=Utn, D=Dl, U=Ul, RRt=ZrZrt, effect=idProvided, record=data$recordF))
 }
 ###
 adjBeta <- function(x){
@@ -460,30 +461,18 @@ redmm <- function (x, M = NULL, Lam=NULL, nPC=50, cholD=FALSE, returnLam=FALSE)
   
 }
 
-imputev <- function (x, method = "median") {
+imputev <- function (x, method = "median", by=NULL) {
   if (is.numeric(x)) {
-    if (method == "mean") {
-      x[which(is.na(x))] <- mean(x, na.rm = TRUE)
-    }
-    else if (method == "median") {
-      x[which(is.na(x))] <- median(x, na.rm = TRUE)
-    }
-    else {
-      x[which(is.na(x))] <- mean(x, na.rm = TRUE)
-    }
-  }
-  else {
-    if (method == "mean") {
-      stop("Method 'mean' is not available for non-numeric vectors.", 
-           call. = FALSE)
-    }
-    else if (method == "median") {
-      tt <- table(x)
-      x[which(is.na(x))] <- names(tt)[which(tt == max(tt))]
-    }
-    else {
-      x[which(is.na(x))] <- names(tt)[which(tt == max(tt))]
+    if(is.null(by)){
+      by <- rep("A",length(x))
     }
+    ms <- aggregate(x~by, FUN=method, na.rm=TRUE)
+    rownames(ms) <- ms$by
+    y <- ms[by,"x"]
+    x[which(is.na(x))] <- y[which(is.na(x))]
+  } else { # if factor
+    tt <- table(x)
+    x[which(is.na(x))] <- names(tt)[which(tt == max(tt))]
   }
   return(x)
 }
@@ -495,8 +484,18 @@ rrm <- function(x=NULL, H=NULL, nPC=2, returnGamma=FALSE, cholD=TRUE){
   # we produce loadings, the Z*L so we can use it to estimate factor scores in mmec()
   
   Y <- apply(H,2, imputev)
-  Sigma <- cov(scale(Y, scale = TRUE, center = TRUE)) # surrogate of unstructured matrix to start with
-  Sigma <- as.matrix(nearPD(Sigma)$mat)
+  Ys <- scale(Y, scale = TRUE, center = TRUE)
+  nans <- which(is.nan(Ys), arr.ind = TRUE)
+  if(nrow(nans) > 0){
+    Ys[nans]=0
+  }
+  Sigma <- cov(Ys) # surrogate of unstructured matrix to start with
+  Sigma <- as.matrix(Matrix::nearPD(x=Sigma, corr = FALSE, keepDiag = FALSE, base.matrix = FALSE,
+                                    do2eigen = TRUE, doSym = FALSE,
+                                    doDykstra = TRUE, only.values = FALSE,
+                                    ensureSymmetry = !isSymmetric(Sigma),
+                                    eig.tol = 1e-06, conv.tol = 1e-07, posd.tol = 1e-08,
+                                    maxit = 100, conv.norm.type = "I", trace = FALSE)$mat)
   # GE <- as.data.frame(t(scale( t(scale(Y, center=T,scale=F)), center=T, scale=F)))  # sum(GE^2)
   if(cholD){
     ## OPTION 2. USING CHOLESKY
