@@ -1,12 +1,15 @@
-## -----------------------------------------------------------------------------
+## ----setup, include=FALSE-----------------------------------------------------
+# knitr::opts_chunk$set(warning = FALSE, message = FALSE) 
 library(lme4breeding)
+
+## -----------------------------------------------------------------------------
 data(DT_example)
 DT <- DT_example
 A <- A_example
 
 ansMain <- lmebreed(Yield ~ Env + (1|Name),
                         relmat = list(Name = A ),
-                        data=DT)
+                        verbose = FALSE, data=DT)
 vc <- VarCorr(ansMain); print(vc,comp=c("Variance"))
 
 
@@ -18,7 +21,7 @@ for(i in 1:ncol(Z)){DT[,colnames(Z)[i]] <- Z[,i]}
 print(as.formula(diagFormula))
 ansDG <- lmebreed(as.formula(diagFormula),
                       relmat = list(Name = A ),
-                      data=DT)
+                      verbose = FALSE, data=DT)
 vc <- VarCorr(ansDG); print(vc,comp=c("Variance"))
 ve <- attr(vc, "sc")^2; ve
 
@@ -31,7 +34,7 @@ colnames(E) <- rownames(E) <- unique(DT$Env);E
 EA <- Matrix::kronecker(E,A, make.dimnames = TRUE)
 ansCS <- lmebreed(Yield ~ Env + (1|Name) + (1|EnvName),
                     relmat = list(Name = A, EnvName=  EA ),
-                    data=DT)
+                    verbose = FALSE, data=DT)
 vc <- VarCorr(ansCS); print(vc,comp=c("Variance"))
 ve <- attr(vc, "sc")^2; ve
 
@@ -42,7 +45,7 @@ for(i in 1:ncol(Z)){DT[,colnames(Z)[i]] <- Z[,i]}
 print(as.formula(csdiagFormula))
 ansCSDG <- lmebreed(as.formula(csdiagFormula),
                       relmat = list(Name = A ),
-                      data=DT)
+                      verbose = FALSE, data=DT)
 vc <- VarCorr(ansCSDG); print(vc,comp=c("Variance"))
 ve <- attr(vc, "sc")^2; ve
 
@@ -54,7 +57,7 @@ for(i in 1:ncol(Z)){DT[,colnames(Z)[i]] <- Z[,i]}
 print(as.formula(usFormula))
 ansDG <- lmebreed(as.formula(usFormula),
                     relmat = list(Name = A ),
-                    data=DT)
+                    verbose = FALSE, data=DT)
 vc <- VarCorr(ansDG); print(vc,comp=c("Variance"))
 ve <- attr(vc, "sc")^2; ve
 
@@ -100,57 +103,68 @@ ei <- ei[with(ei, order(envIndex)), ]
 DT2 <- merge(DT,ei, by="Env")
 DT2 <- DT2[with(DT2, order(Name)), ]
 
-ansFW <- lmebreed(y~ Env + (envIndex || Name), data=DT2)
+ansFW <- lmebreed(y~ Env + (envIndex || Name), verbose = FALSE, data=DT2)
 vc <- VarCorr(ansFW); print(vc,comp=c("Variance"))
 ve <- attr(vc, "sc")^2; ve
 
 
 ## -----------------------------------------------------------------------------
 
-ansFW2 <- lmebreed(y~ Env + (envIndex | Name), data=DT2)
+ansFW2 <- lmebreed(y~ Env + (envIndex | Name), verbose = FALSE, data=DT2)
 vc <- VarCorr(ansFW2); print(vc,comp=c("Variance"))
 ve <- attr(vc, "sc")^2; ve
 
 
 ## -----------------------------------------------------------------------------
 
-# data(DT_h2)
-# DT <- DT_h2
-# DT=DT[with(DT, order(Env)), ]
-# 
-# # fit diagonal model first to produce H matrix
-# Z <- with(DT, smm(Env))
-# diagFormula <- paste0( "y ~ Env + (0+", paste(colnames(Z), collapse = "+"), "|| Name)")
-# for(i in 1:ncol(Z)){DT[,colnames(Z)[i]] <- Z[,i]}
-# print(as.formula(diagFormula))
-# ans1a <- lmebreed(as.formula(diagFormula),
-#                   data=DT)
-# vc <- VarCorr(ans1a); print(vc,comp=c("Variance"))
-# H0 <- ranef(ans1a)$Name # GxE table
-# 
-# # reduced rank model
-# Z <- with(DT,  rrm(Env, H = H0, nPC = 3)) 
-# Zd <- with(DT, smm(Env))
-# faFormula <- paste0( "y ~ Env + (0+", paste(colnames(Z), collapse = "+"), "| Name) + (0+",paste(colnames(Zd), collapse = "+"), "|| Name)")
-# for(i in 1:ncol(Z)){DT[,colnames(Z)[i]] <- Z[,i]}
-# print(as.formula(faFormula))
-# ansFA <- lmebreed(as.formula(faFormula),
-#                   data=DT)
-# vc <- VarCorr(ansFA); print(vc,comp=c("Variance"))
-# ve <- attr(vc, "sc")^2; ve
-# 
-# loadings=with(DT, rrm(Env, nPC = 3, H = H0, returnGamma = T) )$Gamma
-# Gint <- loadings %*% vc$Name %*% t(loadings)
-# Gspec <- diag( unlist(lapply(vc[2:16], function(x){x[[1]]})) )
-# G <- Gint + Gspec
-# # lattice::levelplot(cov2cor(G))
-# # colfunc <- colorRampPalette(c("steelblue4","springgreen","yellow"))
-# # hv <- heatmap(cov2cor(G), col = colfunc(100), symm = TRUE)
-# 
-# u <- ranef(ansFA)$Name
-# uInter <- as.matrix(u[,1:3]) %*% t(as.matrix(loadings))
-# uSpec <- as.matrix(u[,-c(1:3)])
-# u <- uSpec + uInter
+data(DT_h2)
+DT <- DT_h2
+DT=DT[with(DT, order(Env)), ]
+head(DT)
+indNames <- na.omit(unique(DT$Name))
+A <- diag(length(indNames))
+rownames(A) <- colnames(A) <- indNames
+
+# fit diagonal model first to produce H matrix
+Z <- with(DT, smm(Env))
+diagFormula <- paste0( "y ~ Env + (0+", paste(colnames(Z), collapse = "+"), "|| Name)")
+for(i in 1:ncol(Z)){DT[,colnames(Z)[i]] <- Z[,i]}
+ans1a <- lmebreed(as.formula(diagFormula),
+                  relmat = list(Name = A ),
+                  verbose = FALSE,data=DT)
+vc <- VarCorr(ans1a); 
+H0 <- ranef(ans1a)$Name # GxE table
+
+# reduced rank model
+nPC=3
+Z <- with(DT,  rrm(Env, H = H0, nPC = nPC) ) 
+Zd <- with(DT, smm(Env) )
+faFormula <- paste0( "y ~ Env + (0+", paste(colnames(Z), collapse = "+"),
+                     "| Name) + (0+",paste(colnames(Zd), collapse = "+"), "|| Name)")
+for(i in 1:ncol(Z)){DT[,colnames(Z)[i]] <- Z[,i]}
+ansFA <- lmebreed(as.formula(faFormula),
+                  relmat = list(Name = A ),
+                  verbose = FALSE, data=DT)
+
+u <- ranef(ansFA)$Name # all BLUPs
+scores <- as.matrix(u[,1:nPC])  # factor scores
+loadings=with(DT, rrm(Env, nPC = nPC, H = H0, # lodings (latent covars)
+                      returnGamma = TRUE) )$Gamma
+
+vc <- VarCorr(ansFA); print(vc,comp=c("Variance")) # Vcomps
+vcFA <- vc[[1]] # G of PCs
+vcDG <- diag( unlist(lapply(vc[2:16], function(x){x[[1]]})) ) # G of DG
+vcUS <- loadings %*% vcFA %*% t(loadings) # G of US
+G <- vcUS + vcDG # total G
+colfunc <- colorRampPalette(c("steelblue4","springgreen","yellow"))
+hv <- heatmap(cov2cor(G), col = colfunc(100), symm = TRUE)
+
+uFA <- scores %*% t(loadings) # recover US effects
+uDG <- as.matrix(u[,(nPC+1):ncol(u)]) # recover DG effects 
+u <- uFA + uDG # total effects
+
+
+
 
 
 ## -----------------------------------------------------------------------------
@@ -164,7 +178,8 @@ head(DT)
 envs <- unique(DT$Env)
 vals <- list()
 for(i in 1:length(envs)){
-  ans1 <- lmebreed(y~Name + (1|Block), data= droplevels(DT[which(DT$Env == envs[i]),]) )
+  ans1 <- lmebreed(y~Name + (1|Block), verbose = FALSE, 
+                   data= droplevels(DT[which(DT$Env == envs[i]),]) )
   b <- fixef(ans1) 
   b[2:length(b)] <- b[2:length(b)] + b[1]
   ids <- colnames(model.matrix(~Name-1, data=droplevels(DT[which(DT$Env == envs[i]),]) ))
@@ -183,7 +198,7 @@ ans2 <- lmebreed(Estimate~Env + (1|Effect) + (1|Env:Effect), weights = w,data=DT
                    check.nobs.vs.nlev = "ignore",
                    check.nobs.vs.rankZ = "ignore",
                    check.nobs.vs.nRE="ignore"
-                 )
+                 ), verbose = FALSE
                  )
 vc <- VarCorr(ans2); print(vc,comp=c("Variance"))
 ve <- attr(vc, "sc")^2; ve
